@@ -29,41 +29,25 @@
 
 struct dataitem* new_dataitem( )
 {
-	struct dataitem *ret = NULL;
-	
-	ret = malloc( sizeof( struct dataitem ) );
-	if( !ret )
-		return ret;
-	
-	memset( ret, 0, sizeof( struct dataitem ) );
-	
-	return ret;
+	return calloc( 1, sizeof( struct dataitem ) );
 }
 
 void free_dataitem( struct dataitem *freeme )
 {
-	struct dataitem *next = NULL;
 	struct dataitem *this = NULL;
-	
-	if( !freeme )
-		return;
-	
-	if( freeme->humanName )
-		free( freeme->humanName );
-	if( freeme->ac )
-		free( freeme->ac );
-	if( freeme->dataValue )
-		free( freeme->dataValue );
-	
-	this = freeme->next;
-	while( this )
-	{
-		next = this->next;
-		free_dataitem( this );
-		this = next;
+
+	while (freeme) {
+		if (freeme->humanName)
+			free(freeme->humanName);
+		if (freeme->ac)
+			free(freeme->ac);
+		if (freeme->dataValue)
+			free(freeme->dataValue);
+
+		this = freeme->next;
+		free(freeme);
+		freeme = this;
 	}
-	
-	free( freeme );
 }
 
 int calc_packed_length_dataitem( struct dataitem *packme )
@@ -71,39 +55,55 @@ int calc_packed_length_dataitem( struct dataitem *packme )
 	int ret = 3;
 	if( !packme )
 		return 0;
-		
+
 	if( packme->ac )
 		ret += strlen( packme->ac );
 	if( packme->dataValue )
 		ret += strlen( packme->dataValue );
 	if( packme->humanName )
 		ret += strlen( packme->humanName );
-	
+
 	return ret;
 }
 
 struct dataitem * unpack_dataitem( void *buffer )
 {
+	if (buffer == NULL)
+		return NULL;
+
 	char *buf = (char*)buffer;
 	struct dataitem *ret = NULL;
-	
+
 	ret = new_dataitem( );
 	if( !ret )
 		return ret;
-	
+
 	ret->ac = strdup( buf );
+	if (ret->ac == NULL)
+		goto unpackerr;
 	buf += strlen( ret->ac ) + 1;
+
 	ret->humanName = strdup( buf );
+	if (ret->humanName == NULL)
+		goto unpackerr;
 	buf += strlen( ret->humanName ) + 1;
+
 	ret->dataValue = strdup( buf );
+	if (ret->dataValue == NULL)
+		goto unpackerr;
+
 	return ret;
+
+unpackerr:
+	free_dataitem(ret);
+	return NULL;
 }
 
 void add_dataitem( struct dataitem *head, const struct dataitem *addme )
 {
 	if( !head || !addme )
 		return;
-	
+
 	while( head->next )
 		head = head->next;
 	head->next = (struct dataitem *) addme;
